(function($) {
	function speedyAutocomplete(element, options) {
		this.element = element;
		this.options = options;
		this.timer = null;
		this.items = new Array();

		jQuery(element).attr('autocomplete', 'off');
		jQuery(element).on('focus', $.proxy(this.focus, this));
		jQuery(element).on('blur', $.proxy(this.blur, this));
		jQuery(element).on('keydown', $.proxy(this.keydown, this));

		jQuery(element).after('<ul class="dropdown-menu flatsome-speedy-autocomplete ui-autocomplete ui-front ui-menu ui-widget ui-widget-content"></ul>');
		jQuery(element).siblings('ul.dropdown-menu').delegate('a, li', 'click', $.proxy(this.click, this));
	}

	speedyAutocomplete.prototype = {
		focus: function() {
			// this.request();
		},
		blur: function() {
			setTimeout(function(object) {
				object.hide();
			}, 200, this);
		},
		click: function(event) {
			event.preventDefault();

			if (jQuery(event.target).is('a')) {
				value = jQuery(event.target).parent().attr('data-value');
			} else {
				value = jQuery(event.target).attr('data-value');
			}

			if (value && this.items[value]) {
				this.options.select(this.items[value]);
			}
		},
		keydown: function(event) {
			switch(event.keyCode) {
				case 9: // tab
				case 17: // ctr
				case 27: // escape
					this.hide();
					break;
				default:
					this.request();
					break;
			}
		},
		show: function() {
			var pos = jQuery(this.element).position();

			jQuery(this.element).siblings('ul.dropdown-menu').css({
				top: pos.top + jQuery(this.element).outerHeight(),
				left: pos.left
			});

			jQuery(this.element).siblings('ul.dropdown-menu').show();
		},
		hide: function() {
			jQuery(this.element).siblings('ul.dropdown-menu').hide();
		},
		request: function() {
			clearTimeout(this.timer);

			this.timer = setTimeout(function(object) {
				object.options.source(jQuery(object.element).val(), $.proxy(object.response, object));
			}, 200, this);
		},
		response: function(json) {
			html = '';

			if (json.length) {
				for (i = 0; i < json.length; i++) {
					this.items[json[i]['value']] = json[i];
				}

				for (i = 0; i < json.length; i++) {
					if (!json[i]['category']) {
						html += '<li class="ui-menu-item" onMouseOver="this.style.background=\'lightgray\'" onMouseOut="this.style.background=\'white\'" data-value="' + json[i]['value'] + '"><a href="#">' + json[i]['label'] + '</a></li>';

					}
				}

				// Get all the ones with a categories
				var category = new Array();

				for (i = 0; i < json.length; i++) {
					if (json[i]['category']) {
						if (!category[json[i]['category']]) {
							category[json[i]['category']] = new Array();
							category[json[i]['category']]['name'] = json[i]['category'];
							category[json[i]['category']]['item'] = new Array();
						}

						category[json[i]['category']]['item'].push(json[i]);
					}
				}

				for (i in category) {
					html += '<li class="dropdown-header">' + category[i]['name'] + '</li>';

					for (j = 0; j < category[i]['item'].length; j++) {
						html += '<li data-value="' + category[i]['item'][j]['value'] + '"><a href="#">&nbsp;&nbsp;&nbsp;' + category[i]['item'][j]['label'] + '</a></li>';
					}
				}
			}

			if (html) {
				this.show();
			} else {
				this.hide();
			}

			jQuery(this.element).siblings('ul.dropdown-menu').html(html);
		}
	};

	$.fn.speedyAutocomplete = function(option) {
		return this.each(function() {
			var data = jQuery(this).data('autocomplete');

			if (!data) {
				data = new speedyAutocomplete(this, option);

				jQuery(this).data('autocomplete', data);
			}
		});
	}
})(window.jQuery);